#Icons from http://en.wikipedia.org/wiki/Nuvola

from preffile import *
from calendardialog import *
import os
import gobject

RESOURCES_BASEDIR = '/usr/lib/python2.5/site_packages/'
#RESOURCES_BASEDIR = ''

##Pref dialog exception class
class PrefDialogException(Exception):
	def __init__(self, message):
		Exception.__init__(self, message)

##PrefDialog permits to create easily preferences dialog for an application
class PrefDialog(gtk.Window):
	##constructor
	def __init__(self, title=None, parent=None, flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT):
		#init members
		self.__prefDialogSectionIdItems = {}
		self.__prefDialogSectionLabelId = {}
		self.__response = gtk.RESPONSE_CANCEL
		
		#super constructor
		gtk.Window.__init__(self)
		self.set_title(title)
		self.set_modal(True)
		self.resize(600, 300)
		self.connect("delete_event", self.__quit)
				
		#create vbox container (like in gtk.Dialog)
		self.vbox = gtk.VBox(False)
		self.vbox.set_border_width(2)
		self.add(self.vbox)
		
		#add section combo box
		self.__sectionsList = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING)
		self.__sections = gtk.ComboBox(self.__sectionsList)
		cellt = gtk.CellRendererText()
		cellpb = gtk.CellRendererPixbuf()
		self.__sections.pack_start(cellpb, False)
		self.__sections.pack_start(cellt, True)
		self.__sections.set_attributes(cellpb, pixbuf=0)
		self.__sections.set_attributes(cellt, text=1)
		self.vbox.pack_start(self.__sections, False, False)
		self.__sections.connect('changed', self.__onSectionChanged)
		
		#add content container
		self.__scrolled = gtk.ScrolledWindow()
		self.__scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.vbox.pack_start(self.__scrolled, True, True, 2)
		
		#add buttons
		hbox = gtk.HBox(False)
		buttonCancel = gtk.Button(' Cancel ')
		buttonCancel.connect('clicked', self.__onCancel)
		hbox.pack_end(buttonCancel, False, False)
		buttonOk = gtk.Button('    Ok    ')
		buttonOk.connect('clicked', self.__onOk)
		hbox.pack_end(buttonOk, False, False, 2)
		self.vbox.pack_end(hbox, False, False)
		
		#init item container
		self.__content = None

	##Callback when user cancel dialog
	def __onCancel(self, widget, data=None):
		#set response
		self.__response = gtk.RESPONSE_CANCEL
		
		#quit dialog
		self.__quit(widget)
	
	##Callback when user valid dialog
	def __onOk(self, widget, data=None):
		#set response
		self.__response = gtk.RESPONSE_OK
		
		#quit dialog
		self.__quit(widget)
	
	##Close dialog
	def __quit(self, widget, data=None):
		#unlock dialog
		gtk.main_quit()
	
	##Open preferences dialog
	def run(self):
		#select first section
		self.__sections.set_active(0)
		
		#show all widgets
		self.show_all()
		
		#main thread
		gtk.main()
		
		#return response
		return self.__response
	
	##Callback when user change section
	def __onSectionChanged(self, widget, data=None):
		#remove all displayed items
		if self.__content!=None:
			self.__viewport.destroy()
			self.__content.destroy()
			self.realize()
		
		#create new item container
		self.__content = gtk.Table(1,3,False)
		self.__viewport = gtk.Viewport()
		self.__viewport.add(self.__content)
		self.__viewport.set_shadow_type(gtk.SHADOW_NONE)
		self.__scrolled.add(self.__viewport)
	
		#add section items
		self.__displaySectionItems()
	
	##Display items of selected section
	def __displaySectionItems(self):
		#get current section id
		sectionId = self.__sectionsList[self.__sections.get_active()][1]
		
		#add all section items to container
		items = self.__prefDialogSectionIdItems[sectionId]
		line = 0
		for item in items:
			#build item
			(help, label, widget) = item.buildGtkWidget()
			
			#add help
			self.__content.attach(help, 0, 1, line, line+1, gtk.SHRINK, 0, 2 , 2)
			
			#add label
			self.__content.attach(label, 1, 2, line, line+1, gtk.FILL|gtk.SHRINK, 0, 2 , 2)
			
			#add widget
			alignment = gtk.Alignment()
			alignment.add(widget)
			self.__content.attach(alignment, 2, 3, line, line+1, gtk.FILL|gtk.SHRINK, 0, 2 , 2)
			
			line += 1
			
		#show all widgets
		self.show_all()

	##Add section to preferences dialog
	##@param sectionId : unique section id
	##@param sectionLabel : Label to display in combo box for section
	##@param icon : icon to display in combo box
	##@except PrefDialogException : when section id already exists
	def addSection(self, sectionId, sectionLabel, icon=None):
		#check if section id already exists or not
		if self.__prefDialogSectionLabelId.has_key(sectionId):
			#section already exists
			raise PrefDialogException('Section id already exists')
	
		#load icon
		pix = None
		if icon!=None:
			pix = self.__pixBufFromFile(icon)
			
		#save section infos
		self.__prefDialogSectionIdItems[sectionId] = []
		self.__prefDialogSectionLabelId[sectionLabel] = sectionId
			
		#add section
		self.__sectionsList.append([pix, sectionLabel])
	
	##Add item to preferences dialog
	##@param sectionId : add item to specified section id
	##@param PrefDialogItem : item to add in specified section
	##@except PrefDialogException : when specified section id doesn't exist
	def addPrefDialogItem(self, sectionId, prefDialogItem):
		#check if section id exists
		if self.__prefDialogSectionIdItems.has_key(sectionId):
			#section exists, continue
			#add item to section
			self.__prefDialogSectionIdItems[sectionId].append(prefDialogItem)
		else:
			#raise exception
			raise PrefDialogException('Section id doesn\'t exists')
	
	##Add list of items to preferences dialog
	##@see addPrefDialogItem()
	def addPrefDialogItems(self, sectionId, prefDialogItems):
		for prefDialogItem in prefDialogItems:
			self.addPrefDialogItem(prefDialogItem)
	
	##Load image and return pixbuffer
	##@param file : image to load
	def __pixBufFromFile(self, file):
		res = None
		if os.path.exists(file):
			res = gtk.gdk.pixbuf_new_from_file(file)
		return res

##Pref dialog exception class
class PrefDialogItemException(Exception):
	def __init__(self, message):
		Exception.__init__(self,message)

##Abstract class of preference dialog item
class PrefDialogItem():
	##Constants
	HELP_IMAGE = RESOURCES_BASEDIR+'res/help.png'

	##Constructor
    ##@param prefFileItem : preferences file item used to load and save preference value
	##@param label : label of preference item
	def __init__(self, prefFileItem, label):
		#set members
		self.__item = prefFileItem
		self.__help = ''
		self.__label = label
		self.__readOnly = False
		self.widgetWidth = -1

	##Set help message of pref item
	def setHelp(self, help):
		self.__help = help
	
	##Return help message
	def getHelp(self):
		return self.__help
	
	##Set if item is read only or not
	##@param readOnly : True to set item to read only
	def setReadOnly(self, readOnly):
		self.__readOnly = True

	##Return pref file item
	def getPrefFileItem(self):
		return self.__item
		
	##Return pre file item value
	def getPrefFileItemValue(self):
		return self.__item.getValue()
	
	##Set pref file item value
	def setPrefFileItemValue(self, value):
		try:
			#try to set item value (use item validator)
			self.__item.setValue(value)
		except Exception,e :
			#unable to set item value, restore previous one
			self.setWidgetValue(self.getPrefFileItemValue())
	
	##Return Pref file item type
	def getPrefFileItemType(self):
		return self.__item.getType()
	
	##Set widget value
	##Abtract method
	def setWidgetValue(self, value):
		raise Exception('setValue() is not implemented in %s class' % self.__class__.__name__)
	
	##Check if pref file item type is valid
	##Abstract method
	def checkItemType(self, type):
		raise Exception('checkItemType() is not implemented in %s class' % self.__class__.__name__)
	
	##Change pref item width
	def setWidth(self, width):
		self.widgetWidth = width

	##Return GTK widget to be displayed in GTK window
	##Abstract method
	def getGtkWidget(self):
		raise Exception('getGtkWidget() is not implemented in %s class' % self.__class__.__name__)
	
	##Callback when help button is pressed
	def __onShowHelp(self, *args):
		help = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK, self.getHelp())
		help.run()
		help.destroy()
		
	##Build gtk widget to display in PrefDialog
	##@except PrefDialogItemException : when pref item type is not supported
	##@return : list of 3 parts of PrefDialogItem (an help button, a label, the custom widget)
	def buildGtkWidget(self):
		#check item type
		if not self.checkItemType(self.getPrefFileItemType()):
			raise PrefDialogItemException('%s item doesn\'t support %s items' % (self.__class__.__name__, self.getPrefFileItemType()))

		#add description
		if len(self.getHelp())>0:
			help = gtk.Button()
			image = gtk.Image()
			image.set_from_file(self.HELP_IMAGE)
			help.set_image(image)
			help.connect('clicked', self.__onShowHelp)
			help.set_alignment(0.5, 0.5)
			help.set_relief(gtk.RELIEF_NONE)
		else:
			#no description specified, add no text label to keep space
			help = gtk.Label('')

		#add label
		label = gtk.Label(self.__label)
		label.set_alignment(xalign=1, yalign=0.5)

		#add specified widget
		widget = self.getGtkWidget()
		if widget!=None:
			#readonly
			widget.set_sensitive(not self.__readOnly)

		#return widgets
		return help,label,widget

##PrefDialogItem with a checkbox
class PrefDialogItemTrueFalse(PrefDialogItem):
	##Constructor
	def __init__(self, prefFileItem, label):
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Set widget value
	def setWidgetValue(self, value):
		if value:
		    self.__widget.set_active(True)
		else:
			self.__widget.set_active(False)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		if type==PrefFileItem.TYPE_BOOLEAN:
			return True
		else:
			return False

	##Build pref dialog item
	def getGtkWidget(self):
		#build specific widget
		self.__widget = gtk.CheckButton()
		
		#set current value
		self.setWidgetValue(self.getPrefFileItemValue())
		
		#add event
		self.__widget.connect('toggled', self.__changeValue)

		return self.__widget

	##Callback when widget value changed
	def __changeValue(self, *args):
		self.setPrefFileItemValue(self.__widget.get_active())

##PrefDialogItem with an entry
class PrefDialogItemInput(PrefDialogItem):
	##Constructor
	def __init__(self, prefFileItem, label):
	    #members
		self.__widget = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		#input support all types
		return True
	
	##Set widget value
	def setWidgetValue(self, value):
		self.__widget.set_text(str(value))

	##Build pref dialog item
	def getGtkWidget(self):
		#build specific widget
		self.__widget = gtk.Entry()
		
		#set widget width
		if self.widgetWidth>0:
			self.__widget.set_size_request(self.widgetWidth, -1)
		
		#set current value
		self.setWidgetValue(self.getPrefFileItemValue())
		
		#add event
		self.__widget.connect('focus-out-event', self.__changeValue)

		return self.__widget

	##callback when widget value changed
	def __changeValue(self, *args):
		self.setPrefFileItemValue(self.__widget.get_text())

##PrefDialogItem with a combo box
class PrefDialogItemList(PrefDialogItem):
	##Constructor
	def __init__(self, prefFileItem, label):
		#members
		self.__widget = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		#support all types
		return True
	
	##Set combo box options
	def setOptions(self, options):
		#init
		found = False
	
		#get item value
		value = self.getPrefFileItemValue()
	
		#check if value exists in options
		for option in options:
			if option==value:
				#found
				found = True
				#stop statment
				break
		
		if not found:
			raise PrefDialogItemException('Value "%s" not found in options "%s"' % (value, options))
		else:
			#save options
			self.__options = options

	##Set widget value
	def setWidgetValue(self, value):
		#nothing to set, user can't modify widget data
		pass
	
	##Build pref dialog item
	def getGtkWidget(self):
		#init
		index = 0
		value = self.getPrefFileItemValue()
		
		#build specific widget
		self.__widget = gtk.combo_box_new_text()
		
		#set widget width
		if self.widgetWidth>0:
			self.__widget.set_size_request(self.widgetWidth, -1)
		
		#add options
		for i in range(len(self.__options)):
			#get text
			self.__widget.append_text(self.__options[i])
			#save current value index
			if self.__options[i]==value:
				index = i
		#set current value
		self.__widget.set_active(index)
		#add event
		self.__widget.connect('changed', self.__changeValue)

		return self.__widget

	##Callback when widget value changed
	def __changeValue(self, *args):
	    #build value
		value = self.__widget.get_active_text()

		#save it to item
		self.setPrefFileItemValue(value)

##PrefDialogItem with a spin button
class PrefDialogItemPlusMinus(PrefDialogItem):
	##Constructor
	def __init__(self, prefFileItem, label, min, max):
		#set members
		self.__min = 0
		self.__max = 10000
		self.__widget = None
		self.__adjustment = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
		
	##Check pref file item type is compatible
	def checkItemType(self, type):
		if type==PrefFileItem.TYPE_INT or type==PrefFileItem.TYPE_FLOAT:
			return True
		else:
			return False

	##Correct widget value if necessary
	def __getCorrectValue(self, value):
		#get item value
		result = self.__min
	
		#check value
		if str(value).isalpha():
			#it's a string : ko
			result = self.__min
		elif str(value).isdigit():
			#it's a number : ok
			result = value
		elif value<self.__min:
			#value is less than min : ko
			result = self.__min
		elif value>self.__max:
			#value is greater than max : ko
			result = self.__max
		else:
			#ok
			result = self.__min
			
		return result
		
	##Set widget value
	def setWidgetValue(self, value):
		self.__widget.set_value(value)

	##Build pref dialog item
	def getGtkWidget(self):
		#build specific widget
		self.__adjustment = gtk.Adjustment(0, self.__min, self.__max, 1, 1, 0)
		self.__widget = gtk.SpinButton(self.__adjustment, 0, 0)
		
		#set widget width
		if self.widgetWidth>0:
			self.__widget.set_size_request(self.widgetWidth, -1)
		
		#set current value
		self.setWidgetValue(self.__getCorrectValue(self.getPrefFileItemValue()))
		
		#add event
		self.__widget.connect('changed', self.__changeValue)

		return self.__widget

	##Callback when widget value changed
	def __changeValue(self, *args):
		value = self.__getCorrectValue(self.__widget.get_value_as_int())
		self.setPrefFileItemValue(value)

##PrefDialogItem with a button
##When user click on the button, specified callback is launched
class PrefDialogItemButton(PrefDialogItem):
	##Constructor
	##@param buttonLabel : text to put on button
	##@param onClicked : callback when button is clicked. onClicked receives widget and PrefFileItem in parameters
	def __init__(self, prefFileItem, label, buttonLabel, onClicked):
	    #members
		self.__widget = None
		self.__buttonLabel = buttonLabel
		self.__onClicked = onClicked
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
		
	##Check pref file item type is compatible
	def checkItemType(self, type):
		return True

	##Set widget value
	def setWidgetValue(self, value):
		#nothing to set with button
		pass
		
	##Build pref dialog item
	def getGtkWidget(self):
		#build specific widget
		self.__widget = gtk.Button(self.__buttonLabel)
		
		#set widget width
		if self.widgetWidth>0:
			self.__widget.set_size_request(self.widgetWidth, -1)
				
		#connect onClicked event
		self.__widget.connect('clicked', self.__onClicked, self.getPrefFileItem())

		return self.__widget

##PrefDialogItem with a button to select a color
class PrefDialogItemColor(PrefDialogItem):
	##Constructor
	def __init__(self, prefFileItem, label):
	    #members
		self.__widget = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		if type==PrefFileItem.TYPE_COLOR:
			return True
		else:
			return False

	##Set widget value
	def setWidgetValue(self, value):
		if isinstance(value, gtk.gdk.Color):
			#nothing to do
			pass
		elif isinstance(value, str):
			#parse string to color
			value = gtk.gdk.color_parse(value)
		else:
			#incompatible color, set to default
			value = gtk.gdk.Color()
		self.__widget.set_color(value)
		
	##Build pref dialog item
	def getGtkWidget(self):
		#build color button
		self.__widget = gtk.ColorButton()
		self.__widget.set_size_request(-1, 30)
		
		#set current value
		self.setWidgetValue(self.getPrefFileItemValue())
		
		#add event
		self.__widget.connect('color-set', self.__changeValue)

		return self.__widget
	
	##Convert gtk color to string (to_string method doesn't exist yet in pymaemo)
	def __gtkColorToString(self, color):
		return '#%04X%04X%04X' % (color.red, color.green, color.blue)
		
	##Callback when widget value changed
	def __changeValue(self, *args):
		self.setPrefFileItemValue(self.__gtkColorToString(self.__widget.get_color()))

##PrefDialogItem to select a date		
class PrefDialogItemDate(PrefDialogItem):
	##Constants
	DATE_IMAGE = RESOURCES_BASEDIR+'res/date.png'

	##Constructor
	def __init__(self, prefFileItem, label):
	    #members
		self.__widget = None
		self.__entry = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		if type==PrefFileItem.TYPE_DATE:
			return True
		else:
			return False

	##Set widget value
	def setWidgetValue(self, value):
		self.__entry.set_text(str(value))
		
	##Build pref dialog item
	def getGtkWidget(self):
		#build container
		self.__widget = gtk.HBox()
		
		#add entry to display date
		self.__entry = gtk.Entry()
		self.__entry.set_editable(False)
		self.__entry.set_size_request(150, -1)
		self.setWidgetValue(self.getPrefFileItemValue())
		self.__widget.pack_start(self.__entry)
		
		#add button to open calendar
		button = gtk.Button()
		image = gtk.Image()
		image.set_from_file(self.DATE_IMAGE)
		button.set_image(image)
		button.set_alignment(0.5, 0.5)
		button.set_relief(gtk.RELIEF_NONE)
		self.__widget.pack_start(button)
		
		#add event
		button.connect('clicked', self.__onShowCalendar)

		return self.__widget
	
	##Display calendar
	def __onShowCalendar(self, *args):
		#create calendar dialog
		dialog = CalendarDialog('Select date', None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
		
		#set calendar date
		try :
			year,month,day = self.__entry.get_text().split('/')
			dialog.set_date(int(year), int(month), int(day))
		except CalendarDialogException, e:
			#incorrect date, by default calendar is set to current one
			pass
		
		#open dialog
		if dialog.run()==gtk.RESPONSE_OK:
			#get selected date
			year,month,day = dialog.get_date()
			
			#set item value
			value = '%s/%s/%s' % (year,month,day)
			self.setPrefFileItemValue(value)
			
			#set widget value
			self.setWidgetValue(value)
			
		#destroy dialog
		dialog.destroy()

##PRefDialogItem to choose a file
class PrefDialogItemFile(PrefDialogItem):
	##Constants
	FILE_IMAGE = RESOURCES_BASEDIR+'res/folder.png'
	DIALOG_MODE = gtk.FILE_CHOOSER_ACTION_OPEN
	DIALOG_TITLE = 'Select file'

	##Constructor
	def __init__(self, prefFileItem, label):
	    #members
		self.__widget = None
		self.__entry = None
		
		#super constructor
		PrefDialogItem.__init__(self, prefFileItem, label)
	
	##Check pref file item type is compatible
	def checkItemType(self, type):
		if type==PrefFileItem.TYPE_STRING:
			return True
		else:
			return False

	##Set widget value
	def setWidgetValue(self, value):
		self.__entry.set_text(str(value))
		
	##Build pref dialog item
	def getGtkWidget(self):
		#build container
		self.__widget = gtk.HBox()
		
		#add entry to display date
		self.__entry = gtk.Entry()
		self.__entry.set_editable(False)
		if self.widgetWidth>0:
			self.__entry.set_size_request(self.widgetWidth, -1)
		else:
			self.__entry.set_size_request(300, -1)
		self.setWidgetValue(self.getPrefFileItemValue())
		self.__widget.pack_start(self.__entry, True, True)
		
		#add button to open calendar
		button = gtk.Button()
		image = gtk.Image()
		image.set_from_file(self.FILE_IMAGE)
		button.set_image(image)
		button.set_alignment(0.5, 0.5)
		button.set_relief(gtk.RELIEF_NONE)
		self.__widget.pack_start(button, False, False)
		
		#add event
		button.connect('clicked', self.__onSelectFile)

		return self.__widget
	
	##Display file chooser
	def __onSelectFile(self, *args):
		#create file dialog
		dialog = gtk.FileChooserDialog(self.DIALOG_TITLE, None, self.DIALOG_MODE, (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
		
		#set current file
		if len(self.__entry.get_text())>0:
			dialog.set_filename(self.__entry.get_text())
		
		#open dialog
		if dialog.run()==gtk.RESPONSE_OK:
			#get selected filename
			value = dialog.get_filename()
			
			#set widget value
			self.setWidgetValue(value)
			
		#destroy dialog
		dialog.destroy()

##PrefDialogItem to select a directory
class PrefDialogItemDir(PrefDialogItemFile):
	##Constants
	DIALOG_MODE = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER|gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER
	DIALOG_TITLE = 'Select folder'

	##Constructor
	def __init__(self, prefFileItem, label):
		PrefDialogItemFile.__init__(self, prefFileItem, label)
	
